@@ -129,9 +129,26 @@ param_tests! {
     date_normal_leap_year: ok => "2004-02-29", "2004-02-29";
     date_special_100_not_leap: err => "1900-02-29", OutOfRangeDay;
     date_special_400_leap: ok => "2000-02-29", "2000-02-29";
+    date_special_1600ad_leap: ok => "1600-02-29", "1600-02-29";
+    date_special_1200ad_leap: ok => "1200-02-29", "1200-02-29";
+    date_special_1201ad_not_leap: err => "1201-02-29", OutOfRangeDay;
+    date_special_1202ad_not_leap: err => "1202-02-29", OutOfRangeDay;
+    date_special_1203ad_not_leap: err => "1203-02-29", OutOfRangeDay;
+    date_special_1204ad_leap: ok => "1204-02-29", "1204-02-29";
+    date_special_1300ad_not_leap: err => "1300-02-29", OutOfRangeDay;
+    date_special_1400ad_not_leap: err => "1400-02-29", OutOfRangeDay;
+    date_special_1500ad_not_leap: err => "1500-02-29", OutOfRangeDay;
+    date_special_1bc_leap: ok => "0000-02-29", "0000-02-29";
+    date_special_1ad_not_leap: err => "0001-02-29", OutOfRangeDay;
+    date_special_4ad_leap: ok => "0004-02-29", "0004-02-29";
+    date_special_100ad_not_leap: err => "0100-02-29", OutOfRangeDay;
+    date_special_200ad_not_leap: err => "0200-02-29", OutOfRangeDay;
+    date_special_300ad_not_leap: err => "0300-02-29", OutOfRangeDay;
+    date_special_400ad_leap: ok => "0400-02-29", "0400-02-29";
+    date_special_404ad_leap: ok => "0404-02-29", "0404-02-29";
     date_unix_before_watershed: ok => "19999872000", "2603-10-10";
     date_unix_after_watershed: ok => "20044800000", "1970-08-21";
-    date_unix_too_low: err => "-20000000000", DateTooSmall;
+    date_unix_too_low: err => "-62167219200001", DateTooSmall;
 }
 
 #[test]
@@ -144,14 +161,12 @@ fn date_from_timestamp_extremes() {
         Ok(dt) => panic!("unexpectedly valid, {dt}"),
         Err(e) => assert_eq!(e, ParseError::DateTooLarge),
     }
-    match Date::from_timestamp(-30_610_224_000_000, false) {
+    let d = Date::from_timestamp(-62_167_219_200_000, false).unwrap();
+    assert_eq!(d.to_string(), "0000-01-01");
+    match Date::from_timestamp(-62_167_219_200_001, false) {
         Ok(dt) => panic!("unexpectedly valid, {dt}"),
         Err(e) => assert_eq!(e, ParseError::DateTooSmall),
     }
-    let d = Date::from_timestamp(-11_676_096_000 + 1000, false).unwrap();
-    assert_eq!(d.to_string(), "1600-01-01");
-    let d = Date::from_timestamp(-11_673_417_600, false).unwrap();
-    assert_eq!(d.to_string(), "1600-02-01");
     let d = Date::from_timestamp(253_402_300_799_000, false).unwrap();
     assert_eq!(d.to_string(), "9999-12-31");
     match Date::from_timestamp(253_402_300_800_000, false) {
@@ -160,16 +175,26 @@ fn date_from_timestamp_extremes() {
     }
 }
 
+#[test]
+fn date_from_timestamp_special_dates() {
+    let d = Date::from_timestamp(-11_676_096_000 + 1000, false).unwrap();
+    assert_eq!(d.to_string(), "1600-01-01");
+    // check if there is any error regarding offset at the second level
+    // and if rounding down works
+    let d = Date::from_timestamp(-11_676_096_000 + 86399, false).unwrap();
+    assert_eq!(d.to_string(), "1600-01-01");
+    let d = Date::from_timestamp(-11_673_417_600, false).unwrap();
+    assert_eq!(d.to_string(), "1600-02-01");
+}
+
 #[test]
 fn date_watershed() {
     let dt = Date::from_timestamp(20_000_000_000, false).unwrap();
     assert_eq!(dt.to_string(), "2603-10-11");
     let dt = Date::from_timestamp(20_000_000_001, false).unwrap();
     assert_eq!(dt.to_string(), "1970-08-20");
-    match Date::from_timestamp(-20_000_000_000, false) {
-        Ok(d) => panic!("unexpectedly valid, {d}"),
-        Err(e) => assert_eq!(e, ParseError::DateTooSmall),
-    }
+    let dt = Date::from_timestamp(-20_000_000_000, false).unwrap();
+    assert_eq!(dt.to_string(), "1336-03-23");
     let dt = Date::from_timestamp(-20_000_000_001, false).unwrap();
     assert_eq!(dt.to_string(), "1969-05-14");
 }
@@ -462,12 +487,10 @@ fn datetime_watershed() {
     assert_eq!(dt.to_string(), "2603-10-11T11:33:20");
     let dt = DateTime::from_timestamp(20_000_000_001, 0).unwrap();
     assert_eq!(dt.to_string(), "1970-08-20T11:33:20.001000");
-    match DateTime::from_timestamp(-20_000_000_000, 0) {
-        Ok(dt) => panic!("unexpectedly valid, {dt}"),
-        Err(e) => assert_eq!(e, ParseError::DateTooSmall),
-    }
     let dt = DateTime::from_timestamp(-20_000_000_001, 0).unwrap();
     assert_eq!(dt.to_string(), "1969-05-14T12:26:39.999000");
+    let dt = DateTime::from_timestamp(-20_000_000_000, 0).unwrap();
+    assert_eq!(dt.to_string(), "1336-03-23T12:26:40");
 }
 
 #[test]
@@ -1148,6 +1171,7 @@ param_tests! {
     duration_too_short1: err => "", TooShort;
     duration_too_short2: err => "+", TooShort;
     duration_too_short3: err => "P", TooShort;
+    duration_too_short4: err => "+PT", TooShort;
     duration_1y: ok => "P1Y", "P1Y";
     duration_123y: ok => "P123Y", "P123Y";
     duration_123_8y: ok => "P123.8Y", "P123Y292D";
@@ -1162,6 +1186,12 @@ param_tests! {
     duration_fraction2: ok => "P1Y1DT2H0.5S", "P1Y1DT2H0.5S";
     duration_1: ok => "P1DT1S", "P1DT1S";
     duration_all: ok => "P1Y2M3DT4H5M6S", "P1Y63DT4H5M6S";
+    // FIXME: this is current behaviour, but we should probably error on
+    // out-of order durations (not RFC3339 compliant)
+    duration_all_wrong_order: ok => "P3D2M1YT6S5M4H", "P1Y63DT4H5M6S";
+    // FIXME: this is current behaviour, but we should probably error on
+    // repeated units (not RFC3339 compliant)
+    duration_unit_repeated: ok => "P1Y2Y", "P3Y";
     duration: err => "PD", DurationInvalidNumber;
     duration: err => "P1DT1MT1S", DurationTRepeated;
     duration: err => "P1DT1.1M1S", DurationInvalidFraction;
@@ -1169,8 +1199,11 @@ param_tests! {
     duration_invalid_day_unit1: err => "P1X", DurationInvalidDateUnit;
     duration_invalid_day_unit2: err => "P1", DurationInvalidDateUnit;
     duration_time_42s: ok => "00:00:42", "PT42S";
+    duration_time_42s_no_leading_0: ok => "0:00:42", "PT42S";
     duration_time_1m: ok => "00:01", "PT1M";
+    duration_time_1m_no_leading_0: ok => "0:01:00", "PT1M";
     duration_time_1h_2m_3s: ok => "01:02:03", "PT1H2M3S";
+    duration_time_1h_2m_3s_no_leading_0: ok => "1:02:03", "PT1H2M3S";
     duration_time_fraction: ok => "00:01:03.123", "PT1M3.123S";
     duration_time_extra: err => "00:01:03.123x", ExtraCharacters;
     duration_time_timezone: err => "00:01:03x", ExtraCharacters;
@@ -1204,11 +1237,14 @@ param_tests! {
     duration_days_pos: ok => "+1 day", "P1D";
     duration_days_123days: ok => "123days", "P123D";
     duration_days_time: ok => "1 day 00:00:42", "P1DT42S";
+    duration_days_time_no_leading_0: ok => "1 day 1:00:42", "P1DT1H42S";
+    duration_days_time_comma_no_leading_0: ok => "1 day, 1:00:42", "P1DT1H42S";
     duration_days_time_neg: ok => "-1 day 00:00:42", "-P1DT42S";
     duration_exceeds_day: ok => "PT86500S", "P1DT1M40S";
     duration_days_time_too_short: err => "1 day 00:", TooShort;
     duration_days_time_wrong: err => "1 day 00:xx", InvalidCharMinute;
-    duration_days_time_extra: err => "1 day 00:00:00.123 ", InvalidCharTzSign;
+    duration_days_time_extra: err => "1 day 00:00:00.123 ", ExtraCharacters;
+    duration_days_time_more_than_24_hour: err => "1d 24:01:03", DurationHourValueTooLarge;
     duration_overflow: err => "18446744073709551616 day 12:00", DurationValueTooLarge;
     duration_fuzz1: err => "P18446744073709551611DT8031M1M1M1M", DurationValueTooLarge;
     duration_fuzz2: err => "P18446744073709550PT9970442H6R15D1D", DurationValueTooLarge;
